//---------------------------------------------------------------------------
#ifndef TPidH
#define TPidH
//---------------------------------------------------------------------------
#include <SysUtils.hpp>
#include <Controls.hpp>
#include <Classes.hpp>
#include <Forms.hpp>
#include <StdCtrls.hpp>
#include <mmsystem.h>
#include <Dsgnintf.hpp>
//---------------------------------------------------------------------------
enum TPid_var        { TPid_sv,TPid_pv,TPid_error,TPid_integral,TPid_P,TPid_I,
                       TPid_D,TPid_bias,TPid_dead_v,TPid_work,
                       TPid_low,TPid_high };
enum TPid_type       { TPid_manual,TPid_auto };
enum TPid_change     { TPid_soft,TPid_normal };
enum TPid_action     { TPid_direct,TPid_reverse };
//---------------------------------------------------------------------------
//
//
//
//---------------------------------------------------------------------------
class R_TThread;
class R_TThread2;

class PACKAGE R_Timer: public TComponent
{
 friend R_TThread;

 private:
  bool             REnabled;
  int              RInterval;
  TThreadPriority  RPriority;
  R_TThread       *RR_TThread;
  TNotifyEvent     ROnTimer;

 protected:
  void __fastcall SetEnabled(const bool value);
  void __fastcall SetInterval(const int value);
  void __fastcall SetPriority(const TThreadPriority value);
  void __fastcall SetOnTimer(const TNotifyEvent value);
  void __fastcall Timer(void);
  void __fastcall UpdateTimer(void);
 public:
       __fastcall  R_Timer(TComponent *aOwner);
       __fastcall ~R_Timer(void);

 __published:
   __property bool Enabled = { read=REnabled, write=SetEnabled, default=true };
   __property int Interval = { read=RInterval, write=SetInterval, default=100 };
   __property TThreadPriority Priority = { read=RPriority, write=SetPriority };
   __property TNotifyEvent OnTimer = { read=ROnTimer, write=SetOnTimer, default=NULL};
};
//---------------------------------------------------------------------------
//
//
//
//---------------------------------------------------------------------------
class PACKAGE R_TimerDown: public TComponent
{
 friend R_TThread2;

 private:
  bool             REnabled,
                   RStart;
  DWORD            RCount;
  TThreadPriority  RPriority;
  R_TThread2      *RR_TThread;
  TNotifyEvent     ROnCountStart,
                   ROnCountStop,
                   ROnCountCanceled;

 protected:
  void __fastcall SetEnabled(const bool value);
  void __fastcall SetStart(const bool value);
  void __fastcall SetCount(const DWORD value);
  void __fastcall SetPriority(const TThreadPriority value);
  void __fastcall SetOnCountStart(const TNotifyEvent value);
  void __fastcall SetOnCountStop(const TNotifyEvent value);
  void __fastcall SetOnCountCanceled(const TNotifyEvent value);
  void __fastcall Timer(void);
 public:
       __fastcall  R_TimerDown(TComponent *aOwner);
       __fastcall ~R_TimerDown(void);

 __published:
   __property bool  Enabled = { read=REnabled, write=SetEnabled, default=true };
   __property bool  Start = { read=RStart, write=SetStart, default=false };
   __property DWORD Count = { read=RCount, write=SetCount, default=1000 };
   __property TThreadPriority Priority = { read=RPriority, write=SetPriority };
   __property TNotifyEvent OnCountStart = { read=ROnCountStart, write=SetOnCountStart, default=NULL};
   __property TNotifyEvent OnCountStop = { read=ROnCountStop, write=SetOnCountStop, default=NULL};
   __property TNotifyEvent OnCountCanceled = { read=ROnCountCanceled, write=SetOnCountCanceled, default=NULL};
};
//--------------------------------------------------------------------------
class R_TThread : public TThread
{
 public:
  R_Timer                *ojciec;
  virtual      __fastcall  R_TThread(bool CreateSuspended)
                     : TThread(CreateSuspended) {};
  virtual      __fastcall ~R_TThread() {};
  virtual void __fastcall  Execute(void);
};
//--------------------------------------------------------------------------
class R_TThread2 : public TThread
{
 public:
  R_TimerDown            *ojciec;
  virtual      __fastcall  R_TThread2(bool CreateSuspended)
                     : TThread(CreateSuspended) {};
  virtual      __fastcall ~R_TThread2() {};
  virtual void __fastcall  Execute(void);
};
//--------------------------------------------------------------------------
//
//
//
//--------------------------------------------------------------------------
class PACKAGE R_PID : public TComponent
{
 private:
  float           Rsv,           // warto zadana
                  Rpv,           // warto rzeczywista
                  Rerror,        // bd
                  Rintegral,     // caka
                  Rww_P,Rw_P,    // wspczynnik P
                  Rww_I,Rw_I,    // wspczynnik I
                  Rww_D,Rw_D,    // wspczynnik D
                  RBias,         // bias
                  Rdead_v,
                  Rwork,         // wyjcie regulatora
                  Rlow,Rhigh;    //
  float           Rvar,          // zmienna pomocnicza ( stary bd )
                  Rdiff,         // rniczka
                  Rmanual;       // wyjcie podczas pracy rcznej
  bool            Rdead,         // martwa strefa aktywna czy nie
                  Ron;           // wlaczony regulator
  TPid_type       Rtyp;          // typ regulatora
  TPid_change     Rchange;       // typ trybu zmiany pracy regulatora
  TPid_action     Raction;       // wyjscie bezporednie, odwrcone
  R_Timer        *timer;
//  Dodatek regulator dziala dopiero od pewnego bdu
  bool            Rdist_on;      // wczenie wyczenie dziaania dystansu
  float           Rdist;         // dystans
// Safety - podczas bledu regulator ustawia sie w poloenie zadane
  bool            Rsafety_on;    // waczone Safety
  float           Rsafety;       // wyjscie z regulatora podczas bledu
  float           Rsafety_l,
                  Rsafety_h;     // zakres grny,dolny wart.rzeczywistej po ktorej bd
  bool            RBerror;       // wystpi bd na wejciu
  bool            Ralarm_on,     // waczenie alarmowania
                  RBalarm_l,     // wystpi alarm L
                  RBalarm_h;     // wystpi alarm H
  float           Ralarm_l,      // wartoc alarmu L
                  Ralarm_h;      // warto alarmu H
//  Generacja zdarzenia gdy bd jest wikszy ni zakadana warto
  bool            RstatOn,       // wczona generacja zdarzenia
                  RBstat;        // wyspi bd wikszy ni RBstat
  float           Rstat;
  bool            RPickOn;

  TNotifyEvent    ROnError,      // bd - wystpi safety
                  ROnAlarmL,     // Alarm Low
                  ROnAlarmH,     // Alarm High
                  ROnStat,       //
                  ROnPick;       // Timer

 protected:
  void __fastcall Ustaw(void);
  void __fastcall SetRon(bool ww);
  void __fastcall Set_P(float ww);
  void __fastcall Set_I(float ww);
  void __fastcall Set_D(float ww);
  void __fastcall SetType(TPid_type ww);
  void __fastcall SetAlarm(bool ww);
  void __fastcall SetStat(bool ww);
  void __fastcall SetPick(bool ww);
  void __fastcall SetBias(float ww);

  virtual void __fastcall OnTimer(TObject *Sender);
  virtual void __fastcall RError(void);
  virtual void __fastcall RAlarmL(void);
  virtual void __fastcall RAlarmH(void);
  virtual void __fastcall RStat(void);
  virtual void __fastcall RPick(void);
 public:
        __fastcall R_PID(TComponent* Owner);
  float __fastcall GetOut(void);
  bool  __fastcall GetError(void);
  float __fastcall GetVar(TPid_var ww);
  void  __fastcall Reset(void);
 __published:
  __property bool  Enabled= { read=Ron, write=SetRon, default=false };
  __property float Out_L= { read=Rlow, write=Rlow, default=0 };
  __property float Out_H= { read=Rhigh, write=Rhigh, default=100 };
  __property float Out= { read=Rwork };
  __property float SP= { read=Rsv, write=Rsv, default=0 };
  __property float PV= { read=Rpv, write=Rpv, default=0 };
  __property float WP= { read=Rww_P, write=Set_P, default=0 };
  __property float WI= { read=Rww_I, write=Set_I, default=0 };
  __property float WD= { read=Rww_D, write=Set_D, default=0 };
  __property float Bias= { read=RBias, write=SetBias, default=0 };
  __property bool  Dead= { read=Rdead, write=Rdead, default=false };
  __property float DeadValue= { read=Rdead_v, write=Rdead_v, default=0 };
  __property bool  SafetyEnabled= { read=Rsafety_on, write=Rsafety_on, default=false };
  __property float Safety= { read=Rsafety, write=Rsafety, default=0 };
  __property float SafetyL= { read=Rsafety_l, write=Rsafety_l, default=0 };
  __property float SafetyH= { read=Rsafety_h, write=Rsafety_h, default=100 };
  __property bool  DistanceEnabled= { read=Rdist_on, write=Rdist_on, default=false };
  __property float Distance= { read=Rdist, write=Rdist, default=0 };
  __property float Manual= { read=Rmanual, write=Rmanual, default=0 };
  __property bool  AlarmEnabled= { read=Ralarm_on, write=SetAlarm, default=false };
  __property float AlarmL= { read=Ralarm_l, write=Ralarm_l, default=0 };
  __property float AlarmH=  { read=Ralarm_h, write=Ralarm_h, default=100 };
  __property bool  AlarmLIs= { read=RBalarm_l };
  __property bool  AlarmHIs= { read=RBalarm_h };
  __property TPid_type Type= { read=Rtyp, write=SetType, default=TPid_auto };
  __property TPid_change Change= { read=Rchange, write=Rchange, default=TPid_soft };
  __property TPid_action Action= { read=Raction, write=Raction, default=TPid_direct };
  __property bool  StatEnabled= { read=RstatOn, write=SetStat, default=false };
  __property bool  StatError= { read=RBstat };
  __property float StatLimit= { read=Rstat, write=Rstat, default=0 };

  __property bool  PickEnabled= { read=RPickOn, write=SetPick, default=false };

  __property TNotifyEvent OnError= { read=ROnError, write=ROnError };
  __property TNotifyEvent OnAlarmLow= { read=ROnAlarmL, write=ROnAlarmL };
  __property TNotifyEvent OnAlarmHigh= { read=ROnAlarmH, write=ROnAlarmH };
  __property TNotifyEvent OnStat= { read=ROnStat, write=ROnStat };
  __property TNotifyEvent OnPick={ read=ROnPick, write=ROnPick };
};
//----------------------------------------------------------------------------
//
//
//
//----------------------------------------------------------------------------
class PACKAGE R_Rej : public TCustomControl
{
 private:
  TBevel           *bev1;
  bool              VEnabledOn,   // wczony
                    VEnabledHLOut;// wczone ograniczenie Out
  Graphics::TColor  VColorBrush,  // kolor ta
                    VColorBarOut, // kolor baru rodkowego
                    VColorBarPV,
                    VColorBarSP,
                    VColorBarHLOut,
                    VColorSiatki; // kolor siatki
  Graphics::TFont  *VFont1,
                   *VFont2,
                   *VFont3,
                   *VFont4;
  TShape           *bar1,
                   *bar2,
                   *bar3,
                   *bar4,
                   *bar5;
  float             VVarLow,      // Dolna warto
                    VVarHight,    // Grna warto
                    VOut,         // warto wyjcia zaworu
                    VPV,
                    VSP,
                    VOutHight,    // ograniczenie grne Out
                    VOutLow;      // ograniczenie dolne Out
  String            VTextPV,
                    VTextSP,
                    VTextOut;
 protected:
  void __fastcall SetEnabledOn(bool value);
  void __fastcall SetEnabledHLOut(bool value);
  void __fastcall SetColorBrush(TColor newcolor);
  void __fastcall SetColorSiatki(TColor newcolor);
  void __fastcall SetColorBarOut(TColor newcolor);
  void __fastcall SetColorBarHLOut(TColor newcolor);
  void __fastcall SetColorBarPV(TColor newcolor);
  void __fastcall SetColorBarSP(TColor nwecolor);
  void __fastcall SetFontSiatki(Graphics::TFont *newfont);
  void __fastcall SetFontPV(Graphics::TFont *newfont);
  void __fastcall SetFontSP(Graphics::TFont *newfont);
  void __fastcall SetFontOut(Graphics::TFont *newfont);
  void __fastcall SetVarLow(float value);
  void __fastcall SetVarHight(float value);
  void __fastcall SetOut(float value);
  void __fastcall SetOutLow(float value);
  void __fastcall SetOutHight(float value);
  void __fastcall SetPV(float value);
  void __fastcall SetSP(float value);
  void __fastcall Paint(void);
  void __fastcall SetTextPV(String value);
  void __fastcall SetTextSP(String value);
   void __fastcall SetTextOut(String value);
  MESSAGE void __fastcall CMMouseEnter(TMessage &Msg);
 public:
        __fastcall R_Rej(TComponent* Owner);
 __published:
  __property bool    Enabled=     {read=VEnabledOn, write=SetEnabledOn, default=true };
  __property bool    EnabledHLOut={read=VEnabledHLOut, write=SetEnabledHLOut, default=true };
  __property TColor  BrushColor=  {read=VColorBrush, write=SetColorBrush, nodefault };
  __property TColor  SiatkiColor= {read=VColorSiatki, write=SetColorSiatki, nodefault };
  __property TColor  OutBarColor= {read=VColorBarOut, write=SetColorBarOut, nodefault };
  __property TColor  PVBarColor=  {read=VColorBarPV, write=SetColorBarPV, nodefault };
  __property TColor  SPBarColor=  {read=VColorBarSP, write=SetColorBarSP, nodefault };
  __property TColor  OutHLBarColor={read=VColorBarHLOut, write=SetColorBarHLOut, default=clBlue };
  __property Graphics::TFont  *SiatkiFont= {read=VFont1, write=SetFontSiatki };
  __property Graphics::TFont  *PVFont= {read=VFont2, write=SetFontPV };
  __property Graphics::TFont  *SPFont= {read=VFont3, write=SetFontSP };
  __property Graphics::TFont  *OutFont= {read=VFont4, write=SetFontOut };
  __property String  PVText=      {read=VTextPV, write=SetTextPV};
  __property String  SPText=      {read=VTextSP, write=SetTextSP};
  __property String  OutText=     {read=VTextOut, write=SetTextOut};
  __property float   VarLow=      {read=VVarLow, write=SetVarLow, default=0 };
  __property float   VarHight=    {read=VVarHight, write=SetVarHight,default=100 };
  __property float   Out=         {read=VOut, write=SetOut, default=0 };
  __property float   PV=          {read=VPV, write=SetPV, default=0 };
  __property float   SP=          {read=VSP, write=SetSP, default=0 };
  __property float   OutLow=      {read=VOutLow, write=SetOutLow, default=0 };
  __property float   OutHight=    {read=VOutHight,write=SetOutHight, default=100 };

 BEGIN_MESSAGE_MAP
   MESSAGE_HANDLER(CM_MOUSEENTER,TMessage,CMMouseEnter)
 END_MESSAGE_MAP(TCustomControl)
};

#endif
